@@ -19,9 +19,7 @@ NumericVector muP(NumericVector m, NumericVector id,
 
   arma::mat y1(mat.begin(),dim[0],dim[1],false);
  
-  IntegerVector sz(i.size());
   NumericMatrix mu(i.size(),3);
-  NumericMatrix sd(i.size(),3);
   
   for (int j = 0; j < i.size(); j++) { 
     // find coordinates
